@@ -27,10 +27,10 @@ _iter_typevar_ = TypeVar("_iter_typevar_")
 
 @overload
 def i2f(i: None) -> None:
-    ...
+    ... # pragma: no cover
 @overload
 def i2f(i: IntFloat) -> float:
-    ...
+    ... # pragma: no cover
 def i2f(i):
     if type(i) == bool:
         raise ValueError("Use of bool as float not allowed")
@@ -75,6 +75,8 @@ def v2t(
 
 def is_iterable(it: Any) -> bool:
     """Check if a value is Iterable"""
+    if type(it) is str:
+        return False
     try:
         iter(it)
     except:
@@ -162,16 +164,20 @@ class IterableOverride(Iterable[_iter_typevar_], Generic[_iter_typevar_]):
 
 
 class IterTypeMixin(Iterable[_elem_typevar_], Generic[_elem_typevar_]):
+    """Internal collection support
+
+    TODO: Extended internal API documentation.
+    """
     @overload
     def __iter_type__(self,
         type_: Tuple[Type[_iter_typevar_], ...],
     ) -> Generator[_iter_typevar_, None, None]:
-        ...
+        ... # pragma: no cover
     @overload
     def __iter_type__(self,
         type_: Type[_iter_typevar_],
     ) -> Generator[_iter_typevar_, None, None]:
-        ...
+        ... # pragma: no cover
     def __iter_type__(self, type_):
         """Iterate over elems of an Iterable of certain type
 
@@ -187,6 +193,10 @@ class TypedList(
     List[_elem_typevar_], IterTypeMixin[_elem_typevar_],
     Generic[_elem_typevar_],
 ):
+    """An internal list class that allow only elements of certain type.
+
+    TODO: Extended internal API documentation.
+    """
     def __init__(self, iterable: Iterable[_elem_typevar_]=tuple()):
         super().__init__(iterable)
 
@@ -198,7 +208,7 @@ class TypedList(
         Type[_childelem_typevar_],
         Tuple[Type[_childelem_typevar_], ...],
     ]:
-        raise NotImplementedError
+        ... # pragma: no cover
 
     def __add__(self,  # type: ignore[override]
         x: Union[_elem_typevar_, List[_elem_typevar_]],
@@ -311,9 +321,14 @@ class TypedListMapping(
     MutableSequence[_elem_typevar_], MutableMapping[_index_typevar_, _elem_typevar_],
     IterTypeMixin[_elem_typevar_], Generic[_elem_typevar_, _index_typevar_],
 ):
-    """
-    * TypedListMapping assumes not isinstance(Iterable[_elem_typevar], _elem_typevar)
-    * _elem_type_ has to be valid when _typedListMapping.__init__() is called.
+    """An internal collection class that combines a `MutableSequence` with
+    `MutableMapping`
+
+    TODO: Extended internal API documentation.
+
+    API Notes:
+        TypedListMapping assumes not isinstance(Iterable[_elem_typevar], _elem_typevar)
+        _elem_type_ has to be valid when _typedListMapping.__init__() is called.
     """
     T = TypeVar("T")
     class _List(TypedList):
@@ -354,24 +369,24 @@ class TypedListMapping(
         Type[_childelem_typevar_],
         Tuple[Type[_childelem_typevar_], ...],
     ]:
-        raise NotImplementedError
+        ... # pragma: no cover
 
     @property
     @abc.abstractmethod
     def _index_type_(self) -> Type[_index_typevar_]:
-        raise NotImplementedError
+        ... # pragma: no cover
 
     @property
     @abc.abstractmethod
     def _index_attribute_(self) -> str:
-        ...
+        ... # pragma: no cover
 
     @overload
     def __getitem__(self, key: Union[int, _index_typevar_]) -> _elem_typevar_:
-        ...
+        ... # pragma: no cover
     @overload
     def __getitem__(self: _child_class_, key: slice) -> _child_class_:
-        ...
+        ... # pragma: no cover
     def __getitem__(self: _child_class_, # type: ignore[override]
         key: Union[int, slice, _index_typevar_],
     ) -> Union[_elem_typevar_, _child_class_]:
@@ -394,19 +409,21 @@ class TypedListMapping(
     def __setitem__(self,
         key: Union[int, _index_typevar_], value: _elem_typevar_,
     ) -> None:
-        ...
+        ... # pragma: no cover
     @overload
     def __setitem__(self, key: slice, value: Iterable[_elem_typevar_]) -> None:
-        ...
+        ... # pragma: no cover
     def __setitem__(self,
         key: Union[int, slice, _index_typevar_],
         value: SingleOrMulti[_elem_typevar_].T,
     ) -> None:
+        if self._frozen_:
+            raise TypeError("Can't change a frozen list")
         if isinstance(key, int):
             old = self._list_[key]
             try:
                 self._map_.pop(getattr(old, self._index_attribute_))
-            except:
+            except: # pragma: no cover
                 # If the elem is in _list_ it should also be in _map_
                 raise RuntimeError("Internal error")
             # Todo: can we avoid cast to please Pylance
@@ -418,7 +435,7 @@ class TypedListMapping(
             self._list_[key] = value
             key2 = getattr(value, self._index_attribute_)
             self._map_[key2] = cast(_elem_typevar_, value)
-        elif isinstance(key, slice):
+        elif isinstance(key, slice): # pragma: no cover
             raise NotImplementedError(
                 "Assigning to slice of TypedListMapping"
             )
@@ -436,34 +453,45 @@ class TypedListMapping(
     def __delitem__(self,
         key: Union[int, slice, _index_typevar_],
     ) -> None:
+        if self._frozen_:
+            raise TypeError("Can't change a frozen list")
         if isinstance(key, int):
             old = self._list_[key]
             self._map_.pop(getattr(old, self._index_attribute_))
             self._list_.__delitem__(key)
-        elif isinstance(key, slice):
+        elif isinstance(key, slice): # pragma: no cover
             raise NotImplementedError(
                 "Deketing slice of TypedListMapping"
             )
         elif isinstance(key, self._index_type_):
             v = self._map_.pop(key)
             self._list_.remove(v)
-        raise TypeError(
-            f"key has to be of type 'int', 'slice' or '{self._index_type_}'"
-        )
+        else:
+            raise TypeError(
+                f"key has to be of type 'int', 'slice' or '{self._index_type_}'"
+            )
 
     def clear(self) -> None:
+        if self._frozen_:
+            raise TypeError("Can't change a frozen list")
         self._list_.clear()
         self._map_.clear()
 
     def pop(self, # type: ignore[override]
-        key: Optional[_index_typevar_]=None,
+        key: Optional[Union[_index_typevar_, int]]=None,
     ) -> _elem_typevar_:
+        if self._frozen_:
+            raise TypeError("Can't change a frozen list")
         if key is None:
             elem = self._list_.pop()
             self._map_.pop(getattr(elem, self._index_attribute_))
-        else:
+        elif isinstance(key, self._index_type_):
             elem = self._map_.pop(key)
             self._list_.remove(elem)
+        else:
+            assert isinstance(key, int)
+            elem = self._list_.pop(key)
+            self._map_.pop(getattr(elem, self._index_attribute_))
         return elem
 
     def popitem(self):
@@ -481,10 +509,7 @@ class TypedListMapping(
         return len(self._list_)
 
     def __contains__(self, elem: Any) -> bool:
-        try:
-            return getattr(elem, self._index_attribute_) in self._map_
-        except:
-            return False
+        return getattr(elem, self._index_attribute_) in self._map_
 
     def index(self, elem: _elem_typevar_) -> int: # type: ignore[override]
         """
@@ -494,6 +519,8 @@ class TypedListMapping(
         return self._list_.index(elem)
 
     def insert(self, index: int, value: _elem_typevar_) -> None:
+        if self._frozen_:
+            raise TypeError("Can't change a frozen list")
         self._list_.insert(index, value)
         self._map_[getattr(value, self._index_attribute_)] = value
 
@@ -509,6 +536,8 @@ class TypedListMapping(
     def __iadd__(self: _child_class_,
         x: SingleOrMulti[_elem_typevar_].T,
     ) -> _child_class_:
+        if cast("TypedListMapping", self)._frozen_:
+            raise TypeError("Can't change a frozen list")
         cself = cast(TypedListMapping[_elem_typevar_, _index_typevar_], self)
         new: Tuple[_elem_typevar_, ...]
         if isinstance(x, cself._elem_type_):
@@ -537,12 +566,18 @@ class _ListMappingOverride(
     MutableSequence[_elem_typevar_], MutableMapping[_index_typevar_, _elem_typevar_],
     Generic[_elem_typevar_, _index_typevar_],
 ):
+    """A support class for helping subclassing of ListMapping. It allows to subclass
+    a ListMapping class with an element that is a subclass of the parent ListMapping
+    element.
+
+    TODO: Extended internal API documentation.
+    """
     @overload
     def __getitem__(self, key: Union[int, _index_typevar_]) -> _elem_typevar_:
-        ...
+        ... # pragma: no cover
     @overload
     def __getitem__(self: _child_class_, key: slice) -> _child_class_:
-        ...
+        ... # pragma: no cover
     def __getitem__(self: _child_class_, # type: ignore[override]
         key: Union[int, slice, _index_typevar_],
     ) -> Union[_elem_typevar_, _child_class_]:
@@ -552,10 +587,10 @@ class _ListMappingOverride(
     def __setitem__(self,
         key: Union[int, _index_typevar_], value: _elem_typevar_,
     ) -> None:
-        ...
+        ... # pragma: no cover
     @overload
     def __setitem__(self, key: slice, value: Iterable[_elem_typevar_]) -> None:
-        ...
+        ... # pragma: no cover
     def __setitem__(self,
         key: Union[int, slice, _index_typevar_],
         value: SingleOrMulti[_elem_typevar_].T,
@@ -593,6 +628,12 @@ class _ListMappingOverride(
 
 
 class TypedListStrMapping(TypedListMapping[_elem_typevar_, str], Generic[_elem_typevar_]):
+    """TypeListMapping where the _index_type_ is `str`. By default this also take 'name'
+    as default attribute name for the index. This can be overloaded in a subclass if
+    needed.
+
+    TODO: Extended internal API documentation.
+    """
     @property
     def _index_type_(self):
         return str
@@ -608,5 +649,10 @@ class TypedListStrMapping(TypedListMapping[_elem_typevar_, str], Generic[_elem_t
 class ListStrMappingOverride(
     _ListMappingOverride[_elem_typevar_, str], Generic[_elem_typevar_],
 ):
+    """A support class for helping subclassing of TypeListStrMapping analog to
+    `_ListMappingOverride`.
+
+    TODO: Extended internal API documentation.
+    """
     def __getattr__(self, name: str) -> _elem_typevar_:
         return cast(Any, super()).__getattr__(name)
