@@ -39,6 +39,9 @@ def _rect(
     left: float, bottom: float, right: float, top: float, *,
     enclosure: Optional[Union[float, Sequence[float], prp.Enclosure]]=None,
 ) -> geo.Rect:
+    """undocumented deprecated function;
+    see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/39
+    """
     if enclosure is not None:
         if isinstance(enclosure, prp.Enclosure):
             enclosure = enclosure.spec
@@ -61,6 +64,9 @@ def _rect(
 def _via_array(
     left: float, bottom: float, width: float, pitch: float, rows: int, columns: int,
 ):
+    """undocumented deprecated function;
+    see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/39
+    """
     via = geo.Rect.from_size(width=width, height=width)
     xy0 = geo.Point(x=(left + 0.5*width), y=(bottom + 0.5*width))
 
@@ -73,47 +79,48 @@ def _via_array(
 
 
 class _SubLayout(abc.ABC):
+    """Internal `_Layout` support class"""
     @abc.abstractmethod
     def __init__(self):
-        raise RuntimeError("Internal error: called abstract method")
+        ... # pragma: no cover
 
     @property
     @abc.abstractmethod
     def polygons(self) -> Iterable[geo.MaskShape]:
-        raise RuntimeError("Internal error: called abstract method")
+        ... # pragma: no cover
 
     @abc.abstractmethod
-    def dup(self):
-        raise RuntimeError("Internal error: called abstract method")
+    def dup(self) -> "_SubLayout":
+        ... # pragma: no cover
 
     @abc.abstractmethod
     def move(self, *,
         dxy: geo.Point, move_context: Optional[geo.MoveContext]=None,
     ) -> None:
-        raise RuntimeError("Internal error: called abstract method")
+        ... # pragma: no cover
 
     @abc.abstractmethod
     def moved(self, *,
         dxy: geo.Point, move_context: Optional[geo.MoveContext]=None,
     ) -> "_SubLayout":
-        raise RuntimeError("Internal error: called abstract method")
+        ... # pragma: no cover
 
     @abc.abstractmethod
     def rotate(self, *,
         rotation: geo.Rotation, rot_context: Optional[geo.RotationContext]=None,
     ) -> None:
-        raise RuntimeError("Internal error: called abstract method")
+        ... # pragma: no cover
 
     @abc.abstractmethod
     def rotated(self, *,
         rotation: geo.Rotation, rot_context: Optional[geo.RotationContext]=None,
     ) -> "_SubLayout":
-        raise RuntimeError("Internal error: called abstract method")
+        ... # pragma: no cover
 
     @property
     @abc.abstractmethod
     def _hier_strs_(self) -> Generator[str, None, None]:
-        pass
+        ... # pragma: no cover
 
 
 class MaskShapesSubLayout(_SubLayout):
@@ -190,6 +197,7 @@ class MaskShapesSubLayout(_SubLayout):
 
 
 class _InstanceSubLayout(_SubLayout):
+    """Internal `_Layout` support class"""
     def __init__(self, *,
         inst: ckt._CellInstance, origin: geo.Point,
         layoutname: Optional[str], rotation: geo.Rotation,
@@ -231,20 +239,21 @@ class _InstanceSubLayout(_SubLayout):
         return self._layout
 
     @property
-    def boundary(self) -> geo._Rectangular:
+    def boundary(self) -> Optional[geo._Rectangular]:
         l = (
             self.inst.cell.layouts[self.layoutname]
             if self.layoutname is not None
             else self.inst.cell.layout
         )
-        assert l.boundary is not None
-        return l.boundary.rotated(rotation=self.rotation) + self.origin
+        if l.boundary is None:
+            return None
+        else:
+            return l.boundary.rotated(rotation=self.rotation) + self.origin
 
     @property
     def polygons(self) -> Generator[geo.MaskShape, None, None]:
         yield from self.layout.polygons
 
-
     def dup(self):
         return self
 
@@ -270,7 +279,7 @@ class _InstanceSubLayout(_SubLayout):
 
     def rotated(self, *,
         rotation: geo.Rotation, rot_context: Optional[geo.RotationContext]=None,
-    ) -> "_SubLayout":
+    ) -> "_InstanceSubLayout":
         p = rotation*self.origin
         rot = rotation*self.rotation
 
@@ -286,6 +295,7 @@ class _InstanceSubLayout(_SubLayout):
 
 
 class SubLayouts(_util.TypedList[_SubLayout]):
+    """Internal `_Layout` support class"""
     @property
     def _elem_type_(self):
         return _SubLayout
@@ -320,8 +330,8 @@ class SubLayouts(_util.TypedList[_SubLayout]):
                         return True
                 else:
                     return False
-            elif not isinstance(other_sublayout, _InstanceSubLayout):
-                raise AssertionError("Internal error")
+            elif not isinstance(other_sublayout, _InstanceSubLayout): # pragma: no cover
+                raise RuntimeError("Internal error")
         other = tuple(filter(lambda sl: not add2other(sl), other))
 
         if other:
@@ -336,28 +346,44 @@ class SubLayouts(_util.TypedList[_SubLayout]):
 
 
 class _Layout:
+    """A `_Layout` object contains the shapes making up the layout of a design.
+    Contrary to other EDA layout tools all shapes are put on a net or are netless.
+    Netless are only allowed on mask derived from certain primitives.
+
+    `LayoutFactory.new_layout()` needs to be used to generate new layouts.
+
+    Attributes:
+        fab: the factory with which this _layout is created
+        sublayouts: the sublayouts making up this layout
+        boundary: optional boundary of this layout
+    """
     def __init__(self, *,
         fab: "LayoutFactory",
         sublayouts: SubLayouts, boundary: Optional[geo._Rectangular]=None,
     ):
-        assert (
-            isinstance(fab, LayoutFactory)
-            and isinstance(sublayouts, SubLayouts)
-        ), "Internal error"
         self.fab = fab
         self.sublayouts = sublayouts
         self.boundary = boundary
 
     @property
-    def polygons(self) -> Generator[geo.MaskShape, None, None]:
+    def polygons(self) -> Iterable[geo.MaskShape]:
+        """All the `MaskShape` polygons of this layout.
+
+        Typically use case is exporting to a format that has no net information.
+        """
         for sublayout in self.sublayouts:
                 yield from sublayout.polygons
 
     def _net_sublayouts(self, *, net: net_.Net, depth: Optional[int]) -> Generator[
         MaskShapesSubLayout, None, None,
     ]:
+        """Filter polygons; API to be fixed.
+        see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/34
+        """
         for sl in self.sublayouts:
-            if isinstance(sl, _InstanceSubLayout):
+            if isinstance(sl, _InstanceSubLayout): # pragma: no cover
+                # TODO: add code coverage when finalizing API
+                # see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/34
                 assert isinstance(net, ckt._CircuitNet)
                 if depth != 0:
                     for port in net.childports:
@@ -372,24 +398,25 @@ class _Layout:
             elif isinstance(sl, MaskShapesSubLayout):
                 if net == sl.net:
                     yield sl
-            else:
+            else: # pragma: no cover
                 raise AssertionError("Internal error")
 
     def net_polygons(self, net: net_.Net, *, depth: Optional[int]=None) -> Generator[
         geo.MaskShape, None, None
     ]:
-        if not isinstance(net, net_.Net):
-            raise TypeError("net has to be of type 'Net'")
+        """Filter polygons; API to be fixed.
+        see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/34
+        """
         for sl in self._net_sublayouts(net=net, depth=depth):
-            if isinstance(sl, MaskShapesSubLayout):
-                yield from sl.shapes
-            else:
-                yield from sl.polygons
+            yield from sl.shapes
 
     def filter_polygons(self, *,
         net: Optional[net_.Net]=None, mask: Optional[msk._Mask]=None,
         split: bool=False, depth: Optional[int]=None,
     ) -> Generator[geo.MaskShape, None, None]:
+        """Filter polygons; API to be fixed.
+        see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/34
+        """
         if net is None:
             sls = self.sublayouts
         else:
@@ -408,6 +435,7 @@ class _Layout:
                         yield geo.MaskShape(mask=shape.mask, shape=shape2)
 
     def dup(self) -> "_Layout":
+        """Create a duplication of a layout."""
         return _Layout(
             fab=self.fab,
             sublayouts=SubLayouts(sl.dup() for sl in self.sublayouts),
@@ -418,6 +446,15 @@ class _Layout:
         mask: Optional[msk._Mask]=None, net: Optional[net_.Net]=None,
         depth: Optional[int]=None,
     ) -> geo.Rect:
+        """Return the rectangle enclosing selected shapes; filtering of the
+        shapes is done based on the given arguments.
+
+        Arguments:
+            mask: only shapes on this mask are selected
+            net: only shapes on this net are selected
+            depth: when specified only shapes until a certain hierarchy depth
+                are selected.
+        """
         if net is None:
             if depth is not None:
                 raise TypeError(
@@ -437,14 +474,9 @@ class _Layout:
             top=max(bds.top for bds in boundslist),
         )
 
-    def __iadd__(self, other):
+    def __iadd__(self, other: Union["_Layout", _SubLayout, SubLayouts]):
         if self.sublayouts._frozen_:
             raise ValueError("Can't add sublayouts to a frozen 'Layout' object")
-        if not isinstance(other, (_Layout, _SubLayout, SubLayouts)):
-            raise TypeError(
-                "Can only add '_Layout', '_SubLayout' or 'SubLayouts' object to"
-                " a '_Layout' object"
-            )
 
         self.sublayouts += (
             other.sublayouts if isinstance(other, _Layout) else other
@@ -453,28 +485,37 @@ class _Layout:
         return self
 
     @overload
-    def add_primitive(self, *,
-        prim: prm._Primitive,
+    def add_primitive(self, prim: prm._Primitive, *,
         origin: Optional[geo.Point]=None, x: None=None, y: None=None,
         rotation: geo.Rotation=geo.Rotation.R0,
         **prim_params,
     ) -> "_Layout":
-        ...
+        ... # pragma: no cover
     @overload
-    def add_primitive(self, *,
-        prim: prm._Primitive,
+    def add_primitive(self, prim: prm._Primitive, *,
         origin: None=None, x: IntFloat, y: IntFloat,
         rotation: geo.Rotation=geo.Rotation.R0,
         **prim_params,
     ) -> "_Layout":
-        ...
-    def add_primitive(self, *,
-        prim: prm._Primitive,
+        ... # pragma: no cover
+    def add_primitive(self, prim: prm._Primitive, *,
         origin: Optional[geo.Point]=None,
         x: Optional[IntFloat]=None, y: Optional[IntFloat]=None,
         rotation: geo.Rotation=geo.Rotation.R0,
         **prim_params,
     ) -> "_Layout":
+        """Add the layout for a primitive to a layout. It uses the layout
+        generated with `_Layout.layout_primitive()` and places it in the current
+        layout at a specified location and with a specified rotation.
+
+        Arguments:
+            prim: the primitive for which the generate and place the layout
+            prim_params: the parameters for the primitive
+                This is passed to `_Layout.layout_primitive()`.
+            origin or x, y: origin where to place the primitive layout
+            rotation: the rotation to apply on the generated primitive layout
+                before it is placed. By default no rotation is done.
+        """
         if not (prim in self.fab.tech.primitives):
             raise ValueError(
                 f"prim '{prim.name}' is not a primitive of technology"
@@ -486,7 +527,7 @@ class _Layout:
             y = 0.0 if y is None else _util.i2f(y)
             origin = geo.Point(x=x, y=y)
 
-        primlayout = self.fab.new_primitivelayout(prim, **prim_params)
+        primlayout = self.fab.layout_primitive(prim, **prim_params)
         primlayout.rotate(rotation=rotation)
         primlayout.move(dxy=origin)
         self += primlayout
@@ -494,6 +535,8 @@ class _Layout:
 
     def add_maskshape(self, *, net: Optional[net_.Net]=None, maskshape: geo.MaskShape):
         """Add a geometry MaskShape to a _Layout
+
+        This is a low-level layout manipulation method that does not do much checking.
         """
         for sl in self.sublayouts.__iter_type__(MaskShapesSubLayout):
             if sl.net == net:
@@ -508,6 +551,8 @@ class _Layout:
         prim: prm._DesignMaskPrimitive, net: Optional[net_.Net]=None, shape: geo._Shape,
     ):
         """Add a geometry _Shape to a _Layout
+
+        This is a low-level layout manipulation method that does not do much checking.
         """
         self.add_maskshape(
             net=net,
@@ -515,12 +560,26 @@ class _Layout:
         )
 
     def move(self, *, dxy: geo.Point) -> None:
+        """Move the shapes in the layout by the given displacement.
+        This method changes the layout on which this method is called.
+
+        Arguments:
+            dxy: the displacement to apply to all the shapes in this layout
+        """
         move_context = geo.MoveContext()
         for sl in self.sublayouts:
             sl.move(dxy=dxy, move_context=move_context)
+        if self.boundary is not None:
+            self.boundary += dxy
 
     def moved(self, *, dxy: geo.Point,
     ) -> "_Layout":
+        """Return _Layout with all shapes moved by the given displacement.
+        The original layout is not changed,
+
+        Arguments:
+            dxy: the displacement to apply to all the shapes in this layout
+        """
         move_context = geo.MoveContext()
 
         if self.boundary is None:
@@ -536,6 +595,12 @@ class _Layout:
         )
 
     def rotate(self, *, rotation: geo.Rotation):
+        """Rotate the shapes in the layout by the given rotation.
+        This method changes the layout on which this method is called.
+
+        Arguments:
+            rotation: the rotation to apply to all the shapes in this layout
+        """
         rot_context = geo.RotationContext()
 
         if self.boundary is not None:
@@ -546,6 +611,12 @@ class _Layout:
         )
 
     def rotated(self, *, rotation: geo.Rotation):
+        """Return _Layout with all shapes rotated by the given rotation.
+        The original layout is not changed,
+
+        Arguments:
+            rotation: the rotation to apply to all the shapes in this layout
+        """
         rot_context = geo.RotationContext()
 
         if self.boundary is None:
@@ -561,10 +632,18 @@ class _Layout:
         )
 
     def freeze(self):
+        """see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/37"""
         self.sublayouts._freeze_()
 
     @property
     def _hier_str_(self) -> str:
+        """Return a string representing the full hierarchy of the layout.
+        Indentation is used to represent the hierarchy.
+
+        API Notes:
+            This is for debuggin purposes only and user code should not depend
+                on the exact format of this string.
+        """
         return "\n  ".join(("layout:", *(s for s in self._hier_strs_)))
 
     @property
@@ -580,6 +659,15 @@ class _Layout:
 
 
 class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
+    """Support class to generate layout for a `_Primitive`.
+
+    TODO: Proper docs after fixing the API.
+    see https://gitlab.com/Chips4Makers/PDKMaster/-/issues/25
+
+    API Notes:
+        The API is not finalized yet; backwards incompatible changes are still
+            expected.
+    """
     def __init__(self, fab: "LayoutFactory"):
         self.fab = fab
 
@@ -606,7 +694,7 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
     def _WidthSpacePrimitive(self,
         prim: prm._WidthSpacePrimitive, **widthspace_params,
     ) -> _Layout:
-        if len(prim.ports) != 0:
+        if len(prim.ports) != 0: # pragma: no cover
             raise NotImplementedError(
                 f"Don't know how to generate minimal layout for primitive '{prim.name}'\n"
                 f"of type '{prim.__class__.__name__}'"
@@ -682,6 +770,9 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
             ))
         return layout
 
+    def MIMTop(self, prim: prm.MIMTop, **_):
+        raise ValueError("No generation of MIMTop layer; use MIMCapacitor instead")
+
     def Via(self, prim: prm.Via, **via_params) -> _Layout:
         tech = self.tech
 
@@ -707,11 +798,8 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
                 assert bottom_enc == "tall"
                 bottom_enc = enc.tall()
         assert isinstance(bottom_enc, prp.Enclosure)
-        if isinstance(bottom_enc.spec, float):
-            bottom_enc_x = bottom_enc_y = bottom_enc.spec
-        else:
-            bottom_enc_x = bottom_enc.spec[0]
-            bottom_enc_y = bottom_enc.spec[1]
+        bottom_enc_x = bottom_enc.spec[0]
+        bottom_enc_y = bottom_enc.spec[1]
 
         top = via_params["top"]
         top_enc = via_params["top_enclosure"]
@@ -726,11 +814,8 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
                 assert top_enc == "tall"
                 top_enc = enc.tall()
         assert isinstance(top_enc, prp.Enclosure)
-        if isinstance(top_enc.spec, float):
-            top_enc_x = top_enc_y = top_enc.spec
-        else:
-            top_enc_x = top_enc.spec[0]
-            top_enc_y = top_enc.spec[1]
+        top_enc_x = top_enc.spec[0]
+        top_enc_y = top_enc.spec[1]
 
         width = prim.width
         space = via_params["space"]
@@ -873,7 +958,7 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
                         if well_net != net:
                             raise ValueError(
                                 f"Net '{well_net}' for well '{well.name}' of WaferWire"
-                                f" {bottom.name} is different from net '{net.name}''\n"
+                                f" {bottom.name} is different from net '{net}''\n"
                                 f"\tbut implant '{impl.name}' is same type as the well"
                             )
                     else:
@@ -918,7 +1003,8 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
         assert cont_space is not None
         try:
             wire_idx = cont.bottom.index(wire)
-        except ValueError:
+        except ValueError: # pragma: no cover
+            raise NotImplementedError("Resistor connected from the bottom")
             try:
                 wire_idx = cont.top.index(wire)
             except ValueError:
@@ -929,6 +1015,8 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
         else:
             cont_enc = cont.min_bottom_enclosure[wire_idx]
             cont_args = {"bottom": wire, "x": 0.0, "bottom_width": res_width}
+        if (prim.implant is not None) and isinstance(wire, prm.WaferWire):
+            cont_args["bottom_implant"] = prim.implant
         cont_y1 = -0.5*res_height - cont_space - 0.5*cont.width
         cont_y2 = -cont_y1
 
@@ -971,7 +1059,7 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
         # layout.add_wire(net=port1, wire=cont, y=cont_y1, **cont_args)
         # layout.add_wire(net=port2, wire=cont, y=cont_y2, **cont_args)
         x = cont_args.pop("x")
-        _l_cont = self.fab.new_primitivelayout(
+        _l_cont = self.fab.layout_primitive(
             prim=cont, portnets={"conn": port1}, **cont_args
         )
         _l_cont.move(dxy=geo.Point(x=x, y=cont_y1))
@@ -984,7 +1072,7 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
                             rect=msl.shape, top=(-0.5*res_height - self.tech.grid)
                         )
         layout += _l_cont
-        _l_cont = self.fab.new_primitivelayout(
+        _l_cont = self.fab.layout_primitive(
             prim=cont, portnets={"conn": port2}, **cont_args
         )
         _l_cont.move(dxy=geo.Point(x=x, y=cont_y2))
@@ -1012,6 +1100,111 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
 
         return layout
 
+    def MIMCapacitor(self, prim: prm.MIMCapacitor, **mimcapargs) -> _Layout:
+        try:
+            portnets = mimcapargs.pop("portnets")
+        except KeyError:
+            top = prim.ports.top
+            bottom = prim.ports.bottom
+        else:
+            if set(portnets.keys()) != {"top", "bottom"}:
+                raise ValueError(
+                    f"MIMCapacitor '{prim.name}' needs two port nets ('top', 'bottom')"
+                )
+            top = portnets["top"]
+            bottom = portnets["bottom"]
+
+        via = prim.via
+
+        # Params
+        top_width = mimcapargs["width"]
+        top_height = mimcapargs["height"]
+        connect_up = mimcapargs["bottom_connect_up"]
+
+        # TODO: Allow to specify top of the via layer
+        upper_metal = via.top[0]
+        assert isinstance(upper_metal, prm.MetalWire)
+        assert upper_metal.pin is not None
+        upper_pin = upper_metal.pin[0]
+
+        # Compute dimensions
+        if connect_up:
+            bottomvia_outerwidth = (
+                top_width + 2*prim.min_bottomvia_top_space + 2*via.width
+            )
+            bottomvia_outerheight = (
+                top_height + 2*prim.min_bottomvia_top_space + 2*via.width
+            )
+            bottomvia_outerbound = geo.Rect.from_size(
+                width=bottomvia_outerwidth, height=bottomvia_outerheight,
+            )
+
+            idx = via.bottom.index(prim.bottom)
+            enc = via.min_bottom_enclosure[idx].max()
+            bottom_width = bottomvia_outerwidth + 2*enc
+            bottom_height = bottomvia_outerheight + 2*enc
+
+            enc = via.min_top_enclosure[0].max()
+            bottomupper_outerwidth = bottomvia_outerwidth + 2*enc
+            bottomupper_outerheight = bottomvia_outerheight + 2*enc
+            bottomupper_ringwidth = via.width + 2*enc
+
+            topupper_width = (
+                bottomupper_outerwidth - 2*bottomupper_ringwidth - 2*upper_metal.min_space
+            )
+            topupper_height = (
+                bottomupper_outerheight - 2*bottomupper_ringwidth - 2*upper_metal.min_space
+            )
+        else:
+            enc = prim.min_bottom_top_enclosure.max()
+            bottom_width = top_width + 2*enc
+            bottom_height = top_height + 2*enc
+
+            topupper_width = None
+            topupper_height = None
+
+        # Draw the shapes
+        layout = self.fab.new_layout()
+        via_lay = layout.add_primitive(
+            prim=via, bottom=prim.top, portnets={"conn": top},
+            bottom_width=top_width, bottom_height=top_height,
+            top_width=topupper_width, top_height=topupper_height,
+            bottom_enclosure=prim.min_top_via_enclosure,
+        )
+        via_upmbb = via_lay.bounds(mask=upper_metal.mask)
+        layout.add_shape(prim=upper_pin, net=top, shape=via_upmbb)
+
+        shape = geo.Rect.from_size(width=bottom_width, height=bottom_height)
+        layout.add_shape(prim=prim.bottom, net=bottom, shape=shape)
+
+        if connect_up:
+            shape = geo.RectRing(
+                outer_bound=bottomvia_outerbound,
+                rect_width=via.width, min_rect_space=via.min_space,
+            )
+            layout.add_shape(prim=via, net=bottom, shape=shape)
+
+            shape = geo.Ring(
+                outer_bound=geo.Rect.from_size(
+                    width=bottomupper_outerwidth, height=bottomupper_outerheight,
+                ),
+                ring_width=bottomupper_ringwidth,
+            )
+            layout.add_shape(prim=upper_metal, net=bottom, shape=shape)
+            layout.add_shape(prim=upper_pin, net=bottom, shape=shape)
+
+        bottom_space = (
+            prim.min_bottom_space
+            if prim.min_bottom_space is not None
+            else 0.0
+        )
+        layout.boundary = geo.Rect.from_size(
+            width=(bottom_width + bottom_space),
+            height=(bottom_height + bottom_space),
+        )
+
+        return layout
+
     def Diode(self, prim: prm.Diode, **diode_params) -> _Layout:
         try:
             portnets = diode_params.pop("portnets")
@@ -1026,14 +1219,12 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
             an = portnets["anode"]
             cath = portnets["cathode"]
 
-        if prim.min_implant_enclosure is not None:
-            raise NotImplementedError(
-                "Diode layout generation with min_implant_enclosure specified"
-            )
         wirenet_args = {
             "implant": prim.implant,
             "portnets": {"conn": an if prim.implant.type_ == "p" else cath},
         }
+        if prim.min_implant_enclosure is not None:
+            wirenet_args["implant_enclosure"] = prim.min_implant_enclosure
         if prim.well is not None:
             wirenet_args.update({
                 "well": prim.well,
@@ -1164,7 +1355,7 @@ class _PrimitiveLayouter(dsp.PrimitiveDispatcher):
         raise NotImplementedError("layout generation for Bipolar primitive")
 
 
-class MOSFETInstSpec:
+class MOSFETInstSpec: # pragma: no cover
     """Class that provided the spec for the string of transistors generation.
 
     Used by `_CircuitLayouter.transistors_layout()`
@@ -1175,6 +1366,10 @@ class MOSFETInstSpec:
             The inst parameters like l, w, etc with determine the layout of the transistor.
         contact_left, contact_right: whether to place contacts left or right from the
             transistor. This value needs to be the same between two neighbours.
+
+    API Notes:
+        This class is deprecated and will be removed before v1.0.0. See also
+        https://gitlab.com/Chips4Makers/PDKMaster/-/issues/25
     """
     def __init__(self, *,
         inst: ckt._PrimitiveInstance,
@@ -1220,7 +1415,15 @@ class MOSFETInstSpec:
         return self._contact_right
 
 
-class _CircuitLayouter:
+class _CircuitLayouter: # pragma: no cover
+    """_CircuitLayouter is deprecated class and undocumented.
+
+    see https://gitlab.com/Chips4Makers/PDKMaster/-/issues/25 for development of
+    replacement.
+
+    API Notes:
+        This class is deprecated and user code will fail in future.
+    """
     def __init__(self, *,
         fab: "LayoutFactory", circuit: ckt._Circuit, boundary: Optional[geo._Rectangular]
     ):
@@ -1253,7 +1456,7 @@ class _CircuitLayouter:
                     f"Unconnected port(s) {notfound}"
                     f" for inst '{inst.name}' of primitive '{inst.prim.name}'"
                 )
-            l = self.fab.new_primitivelayout(
+            l = self.fab.layout_primitive(
                 prim=inst.prim, portnets=portnets,
                 **inst.params,
             )
@@ -1307,7 +1510,7 @@ class _CircuitLayouter:
                 f"Wire '{wire.name}' does not have exactly one port named 'conn'"
             )
 
-        return self.fab.new_primitivelayout(
+        return self.fab.layout_primitive(
             wire, portnets={"conn": net}, **wire_params,
         )
 
@@ -1435,7 +1638,7 @@ class _CircuitLayouter:
         ...
     @overload
     def place(self, object_: ckt._Instance, *,
-        origin: None=None, x: IntFloat, y: IntFloat,
+        origin: None=None, x: IntFloat=0.0, y: IntFloat=0.0,
         layoutname: Optional[str]=None, rotation: geo.Rotation=geo.Rotation.R0,
     ) -> _Layout:
         ...
@@ -1447,7 +1650,7 @@ class _CircuitLayouter:
         ...
     @overload
     def place(self, object_: _Layout, *,
-        origin: None=None, x: IntFloat, y: IntFloat,
+        origin: None=None, x: IntFloat=0.0, y: IntFloat=0.0,
         layoutname: Optional[str]=None, rotation: geo.Rotation=geo.Rotation.R0,
     ) -> _Layout:
         ...
@@ -1777,7 +1980,7 @@ class _CircuitLayouter:
 
             origin += geo.Point(x=via_x, y=via_y)
 
-        via_lay = self.fab.new_primitivelayout(
+        via_lay = self.fab.layout_primitive(
             portnets={"conn": net}, prim=via, **via_params,
         )
         via_lay.move(dxy=origin)
@@ -1845,29 +2048,70 @@ class _CircuitLayouter:
 
 
 class LayoutFactory:
-    def __init__(self, tech):
-        if not isinstance(tech, tch.Technology):
-            raise TypeError("tech has to be of type Technology")
+    """The user facing class for creating layouts. This class is also a base
+    class on which own factory classes can be built with specific extensions.
+
+    Parameters:
+        tech: the technology for which to create circuits. Created layout may
+            only contain shapes on masks defined by the technology.
+
+    API Notes:
+        The contract for making subclasses has not been finaziled. Backwards
+            incompatible changes are still expected for subclasses of this class.
+    """
+    def __init__(self, *, tech: tch.Technology):
         self.tech = tech
         self.gen_primlayout = _PrimitiveLayouter(self)
 
     def new_layout(self, *,
         sublayouts: Optional[Union[_SubLayout, SubLayouts]]=None,
         boundary: Optional[geo._Rectangular]=None,
-        cls: Type[_Layout]=_Layout
     ):
+        """Create a new layout.
+
+        Arguments:
+            sublayouts: optional list of sublayouts to add to this new layout
+            boundary: optional boundary of the new layout
+        """
         if sublayouts is None:
             sublayouts = SubLayouts()
         if isinstance(sublayouts, _SubLayout):
             sublayouts = SubLayouts(sublayouts)
 
-        return cls(fab=self, sublayouts=sublayouts, boundary=boundary)
+        return _Layout(fab=self, sublayouts=sublayouts, boundary=boundary)
 
-    def new_primitivelayout(self, prim, **prim_params) -> _Layout:
+    def layout_primitive(self, prim: prm._Primitive, **prim_params) -> _Layout:
+        """Create the layout of a `_Primitive` object.
+
+        This will generate a default layout for a given primitive with the
+        provided paramters. This is a default layout
+
+        Arguments:
+            prim: the primitive to create a layout for
+            prim_params: the parameters for the primitive
+
+        API Notes:
+            User code can't depend on the exact layout generated for a certain
+                primitive. Future improvements to the layout generation code
+                may change the resulting layout.
+        """
         prim_params = prim.cast_params(prim_params)
         return self.gen_primlayout(prim, **prim_params)
 
     def new_circuitlayouter(self, *,
         circuit:ckt._Circuit, boundary: Optional[geo._Rectangular],
     ) -> _CircuitLayouter:
+        """Helper class to generate layout corresponding to a given `_Circuit`.
+        The returned layouter will start with an empty layout with optionally a
+        provided boundary. The layouter API can then be used to build up the
+        layout for the circuit.
+
+        Arguments:
+            circuit: the circuit for which to create a layouter
+            boundary: optional boundary of the created layout
+
+        API Notes:
+            The API of the returned layouter is not fixed yet and backwards
+            incompatible changes are still expected.
+        """
         return _CircuitLayouter(fab=self, circuit=circuit, boundary=boundary)
