@@ -1,6 +1,6 @@
 # SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
 import abc
-from typing import Tuple, Dict, Generic, Optional, TypeVar, cast
+from typing import Iterable, Tuple, Dict, Generic, Optional, TypeVar, cast
 
 from ..typing import IntFloat, SingleOrMulti, OptSingleOrMulti
 from .. import _util
@@ -15,6 +15,13 @@ LibraryType = TypeVar("LibraryType", bound="Library")
 
 
 class _Cell(Generic[LibraryType]):
+    """A cell is an element from a `Library` and represents the building blocks
+    of a circuit. A cell may contain one or more circuits and one or more layouts.
+
+    API Notes:
+        User supported ways for creating cells is not fixed. Backwards incompatible
+        changes are still expected.
+    """
     def __init__(self, *, lib: LibraryType, name: str):
         self.lib = lib
         self.name = name
@@ -34,44 +41,74 @@ class _Cell(Generic[LibraryType]):
 
     @property
     def circuit(self):
+        """The default circuit of the cell;' it's the one with the same name as
+        the cell"""
         try:
             return self.circuits[self.name]
         except KeyError:
-            raise ValueError(f"Cell '{self.name}' has not default circuit")
+            raise ValueError(f"Cell '{self.name}' has no default circuit")
 
     @property
     def layout(self):
+        """The default layout of the cell;' it's the one with the same name as
+        the cell"""
         try:
             return self.layouts[self.name]
         except KeyError:
-            raise ValueError(f"Cell '{self.name}' has not default layout")
+            raise ValueError(f"Cell '{self.name}' has no default layout")
 
     def new_circuit(self, *, name: Optional[str]=None):
+        """Create a new empty circuit for the cell.
+
+        Arguments:
+            name: the name of the circuit. If not specified the same name as the
+                cell will be used.
+        """
         if name is None:
             name = self.name
         
-        circuit = self.cktfab.new_circuit(name)
+        circuit = self.cktfab.new_circuit(name=name)
         self.circuits += circuit
         return circuit
 
-    def add_layout(self, *, layout: "lay._Layout", name: Optional[str]=None):
+    def new_layout(self, *,
+        name: Optional[str]=None, boundary: Optional[geo._Rectangular]=None,
+    ):
+        """Create a new empty layout for the cell.
+
+        Arguments:
+            name: the name of the circuit. If not specified the same name as the
+                cell will be used.
+            boundary: optional boundary for the layout
+        """
         if name is None:
             name = self.name
+
+        layout = self.layoutfab.new_layout(boundary=boundary)
         self.layouts += _CellLayout(name=name, layout=layout)
+        return layout
 
     def new_circuitlayouter(self, *,
         name: Optional[str]=None, boundary: Optional[geo._Rectangular]=None,
     ) -> "lay._CircuitLayouter":
+        """Create a circuit layouter for a circuit of the cell.
+
+        Arguments:
+            name: optional name for the circuit
+        API Notes:
+            _CircuitLayouter API is not fixed.
+                see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/25
+        """
         if name is None:
             name = self.name
-        if not isinstance(name, str):
-            raise TypeError("name has to be a string")
-        try:
-            circuit = self.circuits[name]
-        except KeyError:
-            raise ValueError(f"circuit with name '{name}' not present")
+            circuit = self.circuit
+        else:
+            try:
+                circuit = self.circuits[name]
+            except KeyError:
+                raise ValueError(f"circuit with name '{name}' not present")
 
-        layouter = self.lib.layoutfab.new_circuitlayouter(
+        layouter = self.layoutfab.new_circuitlayouter(
             circuit=circuit, boundary=boundary,
         )
         self.layouts += _CellLayout(name=name, layout=layouter.layout)
@@ -120,11 +157,11 @@ class _OnDemandCell(_Cell[LibraryType], abc.ABC, Generic[LibraryType]):
 
     @abc.abstractmethod
     def _create_circuit(self):
-        pass
+        ... # pragma: no cover
 
     @abc.abstractmethod
     def _create_layout(self):
-        pass
+        ... # pragma: no cover
 
 
 class _Cells(_util.TypedListStrMapping[_Cell[LibraryType]], Generic[LibraryType]):
@@ -151,6 +188,13 @@ class _CellLayouts(_util.TypedListStrMapping[_CellLayout]):
 
 
 class RoutingGauge:
+    """
+    API Notes:
+        API for RoutingGause is not fixed. Backwards incompatible changes may still be
+            expected.
+        see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/36
+            code likely to be moved to c4m-flexcell in the future
+    """
     directions = frozenset(("horizontal", "vertical"))
 
     def __init__(self, *,
@@ -201,6 +245,12 @@ class RoutingGauge:
 
 
 class Library:
+    """
+    API Notes:
+        API with global_nets not None is not fully clear yet. Libraries with
+            global_nets not None may get backwards incompatible changes in the
+            future.
+    """
     def __init__(self, *,
         name: str, tech: tch.Technology, cktfab: Optional["ckt.CircuitFactory"]=None,
         layoutfab: Optional["lay.LayoutFactory"]=None,
@@ -209,31 +259,27 @@ class Library:
         self.name = name
         self.tech = tech
         if cktfab is None:
-            cktfab = ckt.CircuitFactory(tech)
+            cktfab = ckt.CircuitFactory(tech=tech)
         self.cktfab = cktfab
         if layoutfab is None:
-            layoutfab = lay.LayoutFactory(tech)
+            layoutfab = lay.LayoutFactory(tech=tech)
         self.layoutfab = layoutfab
 
         if global_nets is not None:
             global_nets2 = cast(Tuple[str, ...], _util.v2t(global_nets))
-            if not all(isinstance(net, str) for net in global_nets2):
-                raise TypeError(
-                    "global_nets has to be None, a string or an iterable of strings"
-                )
             self.global_nets = frozenset(global_nets2)
         else:
             self.global_nets = None
 
-        self.cells = _Cells()
+        self.cells = _Cells[Library]()
     
-    def new_cell(self, *, name: str):
-        cell = _Cell(lib=self, name=name)
+    def new_cell(self, *, name: str) -> _Cell["Library"]:
+        cell = _Cell[Library](lib=self, name=name)
         self.cells += cell
         return cell
 
     @property
-    def sorted_cells(self):
+    def sorted_cells(self) -> Iterable[_Cell["Library"]]:
         cells = set()
         for cell in self.cells:
             if cell not in cells:
@@ -246,6 +292,13 @@ class Library:
 
 
 class StdCellLibrary(Library):
+    """
+    API Notes:
+        API for StdCellLibrary is not fixed. Backwards incompatible changes may still be
+            expected.
+        see: https://gitlab.com/Chips4Makers/PDKMaster/-/issues/36
+            code likely to be moved to c4m-flexcell in the future
+    """
     def __init__(self, *,
         name: str, tech: tch.Technology, cktfab: Optional["ckt.CircuitFactory"]=None,
         layoutfab: Optional["lay.LayoutFactory"]=None,
@@ -261,3 +314,7 @@ class StdCellLibrary(Library):
         self.routinggauge = _util.v2t(routinggauge)
         self.pingrid_pitch = _util.i2f(pingrid_pitch)
         self.row_height = _util.i2f(row_height)
+
+    @property
+    def sorted_cells(self) -> Iterable[_Cell["StdCellLibrary"]]:
+        return cast(Iterable[_Cell["StdCellLibrary"]], super().sorted_cells)
