@@ -3,6 +3,10 @@ from ..technology import edge as edg
 
 
 class EdgeDispatcher:
+    """Dispatch to class method based on type of `_Edge` subclasses.
+
+    This dispatcher follows the common way of working in the `dispatch` module.
+    """
     def __call__(self, edge: edg._Edge, *args, **kwargs):
         classname = edge.__class__.__name__.split(".")[-1]
         return getattr(self, classname, self._pd_unhandled)(edge, *args, **kwargs)
