@@ -2,10 +2,10 @@
 # SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
 from textwrap import dedent, indent
 from itertools import product
-from typing import Set, Tuple, cast
+from typing import Set, Tuple, Optional, cast, overload
 
 from ... import _util, dispatch as dsp
-from ...typing  import GDSLayerSpecDict
+from ...typing  import GDSLayerSpecDict, SingleOrMulti
 from ...technology import (
     mask as msk, primitive as prm, geometry as geo, technology_ as tch,
 )
@@ -129,6 +129,15 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             **_args_gds_layer(prim, gds_layers=self.gds_layers),
         )
 
+    def DeepWell(self, prim: prm.DeepWell):
+        # Treat it as a regular well
+        return _str_create_basic(
+            prim.name, prim.type_+"Well",
+            minsize=prim.min_width, minspace=prim.min_space,
+            minarea=prim.min_area,
+            **_args_gds_layer(prim, gds_layers=self.gds_layers),
+        )
+
     def WaferWire(self, prim: prm.WaferWire):
         return _str_create_basic(
             prim.name, "active",
@@ -181,6 +190,9 @@ class _LayerGenerator(dsp.PrimitiveDispatcher):
             f"{s_indicator})\n"
         )
 
+    def MIMCapacitor(self, prim: prm.MIMCapacitor):
+        return f"# MIMCAP '{prim.name}': '{prim.top}' over '{prim.bottom}'"
+
     def Diode(self, prim: prm.Diode):
         if len(prim.indicator) == 1:
             s_indicator = f"'{prim.indicator[0].name}'"
@@ -411,10 +423,12 @@ class _AnalogGenerator(dsp.PrimitiveDispatcher):
 
 
 class _LibraryGenerator:
-    def __init__(self, tech):
-        assert isinstance(tech, tch.Technology)
+    def __init__(self, *, tech: tch.Technology):
         self.tech = tech
-        self.metals: Tuple[prm.MetalWire, ...] = tuple(tech.primitives.__iter_type__(prm.MetalWire))
+        self.metals: Tuple[prm.MetalWire, ...] = tuple(filter(
+            lambda m: not isinstance(m, prm.MIMTop),
+            tech.primitives.__iter_type__(prm.MetalWire),
+        ))
         self.vias: Tuple[prm.Via, ...] = tuple(tech.primitives.__iter_type__(prm.Via))
         assert len(self.metals) == len(self.vias)
         self.pinmasks = pinmasks = {}
@@ -427,13 +441,36 @@ class _LibraryGenerator:
                 for p in cast(Tuple[prm.Marker], prim.pin)
             })
 
-    def __call__(self, lib):
-        assert isinstance(lib, lbr.Library)
-        return "\n".join((
+        self.metalsdir: Optional[Tuple[Optional[str], ...]] = None
+
+    def __call__(self, lib: lbr.Library, *, routinggauge: Optional[lbr.RoutingGauge]):
+        def otherdir(dir_):
+            return "vertical" if dir_ == "horizontal" else "horizontal"
+
+        if isinstance(lib, lbr.StdCellLibrary):
+            assert routinggauge is None
+            routinggauge = lib.routinggauge[0]
+
+        if routinggauge is not None:
+            bottom_idx = self.metals.index(routinggauge.bottom)
+            bottom_dir = routinggauge.bottom_direction
+            self.metalsdir = tuple(
+                None if i < (bottom_idx - 1)
+                else bottom_dir if ((i - bottom_idx)%2) == 0
+                else otherdir(bottom_dir) # This is also for bottom_idx - 1
+                for i in range(len(self.metals))
+            )
+
+
+        s = "\n".join((
             self._s_head(), self._s_routing(lib), self._s_load(lib),
             self._s_setup(lib),
         ))
 
+        self.metalsdir = None
+
+        return s
+
     def _s_head(self):
         return dedent(f"""
             # Autogenerated file. Changes will be overwritten.
@@ -442,7 +479,7 @@ class _LibraryGenerator:
             from Hurricane import (
                 Technology, DataBase, DbU, Library,
                 Layer, BasicLayer,
-                Cell, Net, Vertical, Rectilinear, Box, Point,
+                Cell, Net, Horizontal, Vertical, Rectilinear, Box, Point,
                 Instance, Transformation,
                 NetExternalComponents,
             )
@@ -496,8 +533,6 @@ class _LibraryGenerator:
         return s
 
     def _s_gauge(self, lib):
-        def otherdir(dir_):
-            return "vertical" if dir_ == "horizontal" else "horizontal"
         def s_cordir(dir_):
             return (
                 "CRL.RoutingLayerGauge.Horizontal" if dir_ == "horizontal"
@@ -521,15 +556,13 @@ class _LibraryGenerator:
             for i, metal in enumerate(self.metals)
         )
         for i in range(max(bottom_idx - 1, 0), top_idx+1):
-            routedir = rg.bottom_direction
-            if i < bottom_idx:
-                s_usage = "CRL.RoutingLayerGauge.PinOnly"
-                routedir = otherdir(routedir)
-            else:
-                s_usage = "CRL.RoutingLayerGauge.Default"
-                # Take opposite direction for offset of layer number
-                if ((i - bottom_idx) % 2) == 1:
-                    routedir = otherdir(routedir)
+            assert self.metalsdir is not None
+            routedir = self.metalsdir[i]
+            assert routedir is not None
+            s_usage = (
+                "CRL.RoutingLayerGauge.PinOnly" if i < bottom_idx
+                else "CRL.RoutingLayerGauge.Default"
+            )
 
             metal = self.metals[i]
             mwidth = round(mwidths[i], 6)
@@ -828,6 +861,8 @@ class _LibraryGenerator:
             return f"# Export failed for cell '{cell.name}'"
 
     def _s_shape(self, shape: geo.MaskShape) -> str:
+        metalmasks = tuple(metal.mask for metal in self.metals)
+
         mask = shape.mask
         s = ""
 
@@ -836,6 +871,7 @@ class _LibraryGenerator:
 
             if mask in self.pinmasks:
                 metalmask = self.pinmasks[mask]
+                metalidx = metalmasks.index(metalmask)
                 if len(coords) != 5:
                     raise NotImplementedError(
                         f"Non-rectangular pin with coords '{coords}'"
@@ -846,20 +882,49 @@ class _LibraryGenerator:
                 right = max(xs)
                 bottom = round(min(ys), 6)
                 top = round(max(ys), 6)
-                x = round(0.5*(left + right), 6)
                 width = round(right - left, 6)
-                s += dedent(f"""
-                    Vertical.create(
-                        net, tech.getLayer('{mask.name}'),
-                        u({x}), u({width}), u({bottom}), u({top}),
+                height = round(top - bottom, 6)
+
+                routingdir = (
+                    None if self.metalsdir is None
+                    else self.metalsdir[metalidx]
+                )
+                if routingdir is None:
+                    routingdir = (
+                        "vertical" if height > width
+                        else "horizontal"
                     )
-                    pin = Vertical.create(
-                        net, tech.getLayer('{metalmask.name}'),
-                        u({x}), u({width}), u({bottom}), u({top}),
-                    )
-                    net.setExternal(True)
-                    NetExternalComponents.setExternal(pin)
-                """[1:])
+
+                if routingdir == "vertical":
+                    x = round(0.5*(left + right), 6)
+                    s += dedent(f"""
+                        Vertical.create(
+                            net, tech.getLayer('{mask.name}'),
+                            u({x}), u({width}), u({bottom}), u({top}),
+                        )
+                        pin = Vertical.create(
+                            net, tech.getLayer('{metalmask.name}'),
+                            u({x}), u({width}), u({bottom}), u({top}),
+                        )
+                        net.setExternal(True)
+                        NetExternalComponents.setExternal(pin)
+                    """[1:])
+                else:
+                    assert routingdir == "horizontal"
+                    y = round(0.5*(bottom + top), 6)
+                    s += dedent(f"""
+                        Horizontal.create(
+                            net, tech.getLayer('{mask.name}'),
+                            u({y}), u({height}), u({left}), u({right}),
+                        )
+                        pin = Horizontal.create(
+                            net, tech.getLayer('{metalmask.name}'),
+                            u({y}), u({height}), u({left}), u({right}),
+                        )
+                        net.setExternal(True)
+                        NetExternalComponents.setExternal(pin)
+                    """[1:])
+
             else:
                 s += dedent(f"""
                     createRL(
@@ -896,7 +961,7 @@ class _TechnologyGenerator:
             from Hurricane import (
                 Technology, DataBase, DbU, Library,
                 Layer, BasicLayer,
-                Cell, Net, Vertical, Rectilinear, Box, Point,
+                Cell, Net, Horizontal, Vertical, Rectilinear, Box, Point,
                 NetExternalComponents,
             )
             from common.colors import toRGB
@@ -963,7 +1028,8 @@ class _TechnologyGenerator:
                 # Handled by Via
                 prm.WaferWire, prm.GateWire, prm.MetalWire,
                 # Handled later
-                prm.Resistor, prm.Diode, prm.MOSFETGate, prm.MOSFET, prm.Bipolar,
+                prm.Resistor, prm.MIMCapacitor, prm.Diode,
+                prm.MOSFETGate, prm.MOSFET, prm.Bipolar,
                 # Not exported
                 prm._RulePrimitive,
             )):
@@ -1032,6 +1098,12 @@ class _TechnologyGenerator:
             s_prims += gen(prim)
             written_prims.add(prim)
 
+        s_prims += "\n# Capacitors\n"
+        for prim in self.tech.primitives.__iter_type__(prm.MIMCapacitor):
+            assert prim not in written_prims
+            s_prims += gen(prim)
+            written_prims.add(prim)
+
         s_prims += "\n# Transistors\n"
         for prim in self.tech.primitives.__iter_type__((prm.MOSFETGate, prm.MOSFET)):
             assert prim not in written_prims
@@ -1053,7 +1125,7 @@ class _TechnologyGenerator:
             analogTechnologyTable = (
                 ('Header', '{self.tech.name}', DbU.UnitPowerMicro, 'alpha'),
                 ('PhysicalGrid', {self.tech.grid}, Length, ''),
-            
+
             """[1:]
         )
         s += indent(
@@ -1225,11 +1297,28 @@ class FileExporter:
         self.tech = tech
         self.gds_layers = gds_layers
 
-    def __call__(self, obj=None, *, supply="vdd", ground="vss"):
+    @overload
+    def __call__(self,
+        obj: None, *, routinggauge: None=None,
+    ):
+        ...
+    @overload
+    def __call__(self,
+        obj: lbr.StdCellLibrary, *, routinggauge: None=None,
+    ):
+        ...
+    @overload
+    def __call__(self,
+        obj: lbr.Library, *, routinggauge: Optional[lbr.RoutingGauge]=None,
+    ):
+        ...
+    def __call__(self,
+        obj: Optional[lbr.Library]=None, *, routinggauge: Optional[lbr.RoutingGauge]=None,
+    ):
         if obj is None:
             s = self._s_tech()
         elif isinstance(obj, lbr.Library):
-            s = self._s_lib(obj, supply=_util.v2t(supply), ground=_util.v2t(ground))
+            s = self._s_lib(obj, routinggauge=routinggauge)
         else:
             raise TypeError("object has to be None or of type 'Library'")
 
@@ -1239,6 +1328,6 @@ class FileExporter:
         gen = _TechnologyGenerator()
         return gen(tech=self.tech, gds_layers=self.gds_layers)
 
-    def _s_lib(self, lib, *, supply, ground):
-        gen = _LibraryGenerator(self.tech)
-        return gen(lib)
+    def _s_lib(self, lib: lbr.Library, *, routinggauge: Optional[lbr.RoutingGauge]):
+        gen = _LibraryGenerator(tech=self.tech)
+        return gen(lib, routinggauge=routinggauge)
