@@ -222,11 +222,11 @@ class _RuleConverter(dsp.RuleDispatcher):
             if prop == "enclosed_by":
                 # TODO: Proper typing for Property
                 assert isinstance(right, prp.Enclosure)
-                if isinstance(right.spec, float):
+                if not right.is_assymetric:
                     return dedent(f"""
-                        {s_mask2}.enclosing({s_mask1}, {right.spec}).output(
+                        {s_mask2}.enclosing({s_mask1}, {right.first}).output(
                             "{s_mask2}:{s_mask1} enclosure",
-                            "Minimum enclosure of {s_mask2} around {s_mask1}: {right.spec}µm"
+                            "Minimum enclosure of {s_mask2} around {s_mask1}: {right.first}µm"
                         )
                     """[1:])
                 else:
@@ -644,6 +644,13 @@ class _ShapeExporter(dsp.ShapeDispatcher):
         self._mps_exported: Optional[Set[geo.MultiPartShape]]
         self._mps_exported = set() if export_fullshape else None
 
+    def _pointsshapes(self, shape: geo._Shape) -> Generator[Any, None, None]:
+        # Helper to convert the individual pointsshapes
+        for pointshape in shape.pointsshapes:
+            conv = self(pointshape)
+            assert not _util.is_iterable(conv), "Internal error: unsupported"
+            yield conv
+
     def _Shape(self, shape: geo._Shape): # pragma: no cover
         raise ValueError(f"Unsupported object of type {shape.__class__.__name__}")
 
@@ -664,6 +671,10 @@ class _ShapeExporter(dsp.ShapeDispatcher):
     def Rect(self, rect: geo.Rect) -> "pya.DBox": # type: ignore
         return pya.DBox(rect.left, rect.bottom, rect.right, rect.top)
 
+    def RectRing(self, rs: geo.RectRing) -> Generator[Any, None, None]:
+        # TODO: Can repetition information be retained in KLayout ?
+        return self._pointsshapes(rs)
+
     def MultiPartShape(self, mps: geo.MultiPartShape):
         if self._mps_exported is None:
             return self(mps.fullshape)
@@ -690,10 +701,7 @@ class _ShapeExporter(dsp.ShapeDispatcher):
 
     def RepeatedShape(self, rs: geo.RepeatedShape) -> Generator[Any, None, None]:
         # TODO: Can repetition information be retained in KLayout ?
-        for pointshape in rs.pointsshapes:
-            conv = self(pointshape)
-            assert not _util.is_iterable(conv), "Internal error: unsupported"
-            yield conv
+        return self._pointsshapes(rs)
 
     # TODO: Does KLayout allow more efficient array representation
     # ArrayShape -> RepeatedShape
