@@ -2,8 +2,10 @@
 from ... import _util
 from .skill_grammar import SkillFile, _skill_functionlist
 
+
 __all__ = ["TechFile"]
 
+
 #
 # Technology file support functions
 #
@@ -21,10 +23,12 @@ def _get_layername(v):
     else:
         raise ValueError("{!r} is not a valid layer specification")
 
+
 def _get_bool(v):
     assert type(v) is bool
     return v
 
+
 def _get_combinedlayername(v):
     """Returns a combination of layers, the layers will be separated by a ':' as delimiter.
 
@@ -32,6 +36,7 @@ def _get_combinedlayername(v):
     assert len(v) > 1
     return ":".join([_get_layername(elem) for elem in v])
 
+
 def _get_numornil(v):
     if isinstance(v, bool):
         assert not v
@@ -40,6 +45,7 @@ def _get_numornil(v):
         assert isinstance(v, (int, float))
         return v
 
+
 def _prop_value(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -54,6 +60,7 @@ def _prop_value(elems, **kwargs):
 
     return ret
 
+
 def _prop_layers_value_optextra(elems, **kwargs):
     """data: [prop (layer1 (layer2 (layer3))) value (extraprop (extravalue))]
     if no layer is present "_" will be used for layername"""
@@ -90,6 +97,7 @@ def _prop_layers_value_optextra(elems, **kwargs):
 
     return ret
 
+
 def _prop_cumulative_value(elems, *, functionname, **kwargs):
     d = {}
     for v in elems:
@@ -99,13 +107,14 @@ def _prop_cumulative_value(elems, *, functionname, **kwargs):
         elif functionname == "cumulativeViaAntenna":
             cond += ".cumulative_via"
         else:
-            raise Exception("unhandled name '{}'".format(functionname))
+            raise Exception(f"unhandled name '{functionname}'")
         d[cond] = v[1]
         if len(v) > 2:
             d[cond + "." + v[2]] = True if len(v) == 3 else v[3]
 
     return {"_": d}
 
+
 def _prop_value_units(elems, **kwargs):
     ret = {}
     for prop, units, value in elems:
@@ -113,6 +122,7 @@ def _prop_value_units(elems, **kwargs):
 
     return ret
 
+
 def _layer_value(elems, *, unique=False, **kwargs):
     ret = {}
     for layer, value in elems:
@@ -123,6 +133,7 @@ def _layer_value(elems, *, unique=False, **kwargs):
 
     return ret
 
+
 def _layer_values(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -130,12 +141,15 @@ def _layer_values(elems, **kwargs):
 
     return ret
 
+
 def _layers(elems, **kwargs):
     return [_get_layername(elem) for elem in elems]
 
+
 def _combinedlayers(elems, **kwargs):
     return [_get_combinedlayername(elem) for elem in elems]
 
+
 def _techParams(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -153,6 +167,7 @@ def _techParams(elems, **kwargs):
 
     return ret
 
+
 def _name_abbreviation(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -165,6 +180,7 @@ def _name_abbreviation(elems, **kwargs):
 
     return ret
 
+
 def _techDisplays(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -180,6 +196,7 @@ def _techDisplays(elems, **kwargs):
 
     return ret
 
+
 def _standardViaDefs(elems, **kwargs):
     ret = []
     for v in elems:
@@ -227,6 +244,7 @@ def _standardViaDefs(elems, **kwargs):
 
     return ret
 
+
 def _customViaDefs(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -243,6 +261,7 @@ def _customViaDefs(elems, **kwargs):
 
     return ret
 
+
 def _spacingTables(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -287,6 +306,7 @@ def _spacingTables(elems, **kwargs):
 
     return ret
 
+
 def _viaSpecs(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -299,6 +319,7 @@ def _viaSpecs(elems, **kwargs):
 
     return ret
 
+
 def _antennaModels(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -311,7 +332,7 @@ def _antennaModels(elems, **kwargs):
                 elif rulename in ("cumulativeMetalAntenna", "cumulativeViaAntenna"):
                     v2 =  _prop_cumulative_value(spec, functionname=rulename)
                 else:
-                    raise ValueError("Unsupported rulename '{}' for antenneModels".format(rulename))
+                    raise ValueError(f"Unsupported rulename '{rulename}' for antenneModels")
                 for layer, layerspec in v2.items():
                     if layer in rules:
                         rules[layer].update(layerspec)
@@ -321,6 +342,7 @@ def _antennaModels(elems, **kwargs):
 
     return ret
 
+
 def _constraintGroups(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -391,6 +413,7 @@ def _constraintGroups(elems, **kwargs):
 
     return ret
 
+
 def _techDerivedLayers(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -410,6 +433,7 @@ def _techDerivedLayers(elems, **kwargs):
 
     return ret
 
+
 def _functions(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -422,6 +446,7 @@ def _functions(elems, **kwargs):
 
     return ret
 
+
 def _multipartPathTemplates(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -551,6 +576,7 @@ def _multipartPathTemplates(elems, **kwargs):
 
     return ret
 
+
 def _streamLayers(elems, **kwargs):
     ret = {}
     for layer, number, datatype, translate in elems:
@@ -563,6 +589,7 @@ def _streamLayers(elems, **kwargs):
 
     return ret
 
+
 def _layerFunctions(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -576,6 +603,7 @@ def _layerFunctions(elems, **kwargs):
 
     return ret
 
+
 def _spacingRules(elems, **kwargs):
     ret = {}
     for v in elems:
@@ -593,6 +621,7 @@ def _spacingRules(elems, **kwargs):
 
     return ret
 
+
 def _layerDefinitions(elems, **kwargs):
     class _LookupExtend():
         def __init__(self, data):
@@ -652,9 +681,10 @@ def _layerDefinitions(elems, **kwargs):
         return value
 
     d_elems["layers"] = [_layerpurpose2value(layerpurpose) for layerpurpose in techlayerpurposes]
-    assert len(techlayerproperties) == 0, "Remaining layerproperties: {}".format(list(techlayerproperties.keys()))
+    assert len(techlayerproperties) == 0, f"Remaining layerproperties: {list(techlayerproperties.keys())}"
     return d_elems
 
+
 _value4function_table = {
     "techLayerPurposePriorities": _layers,
     "controls": _skill_functionlist,
@@ -755,6 +785,6 @@ class TechFile(SkillFile):
 
     @classmethod
     def parse_string(cls, text):
-        return super(TechFile, cls).parse_string(
+        return super().parse_string(
             text, value4funcs=_value4function_table, dont_convert=["currentDensity"]
         )
