@@ -175,7 +175,7 @@ class _PrimitiveGenerator(dsp.PrimitiveDispatcher):
             else:
                 s += ",\n"
         if prim.model is not None:
-            s += f"model='{prim.model}', model_params={prim.model_params},\n"
+            s += f"model='{prim.model}', subckt_params={prim.subckt_params},\n"
         if prim.sheetres is not None:
             s += f"sheetres={prim.sheetres},\n"
         s += self._params_widthspace(prim)
