@@ -2,18 +2,19 @@
 import abc
 from typing import Union
 
-from ..typing import SingleOrMulti
-from .. import _util
+from ..typing import MultiT, cast_MultiT
 from . import property_ as prp, mask as msk
 
 
-__all__ = ["MaskEdge"]
+__all__ = ["MaskEdge", "Join", "Intersect"]
 
 
 _MaskOrEdge = Union[msk._Mask, "_Edge"]
 
 
 class _EdgeProperty(prp.Property):
+    """`_EdgeProperty` is a `Property` for a `_Edge` object.
+    """
     def __init__(self, *, edge: "_Edge", name: str):
         super().__init__(str(edge) + "." + name)
         self.edge = edge
@@ -21,6 +22,9 @@ class _EdgeProperty(prp.Property):
 
 
 class _DualEdgeProperty(prp.Property):
+    """`_EdgeProperty` is a `Property` related to an `_Edge` object
+    and a second `_Edge` or `Mask` object.
+    """
     def __init__(self, *,
         edge1: "_Edge", edge2: _MaskOrEdge, name: str, commutative: bool, allow_mask2: bool,
     ):
@@ -40,6 +44,11 @@ class _DualEdgeProperty(prp.Property):
 
 
 class _Edge(abc.ABC):
+    """_Edge is a base class representing the edges of shape drawn on a mask.
+    It is used to define it's own operation with their own semantics used in
+    rules. For example intersection of edges has a different meaning than the
+    intersecton of shapes on a mask.
+    """
     @abc.abstractmethod
     def __init__(self, *, name: str):
         self.name = name
@@ -49,19 +58,22 @@ class _Edge(abc.ABC):
     def __str__(self):
         return self.name
 
-    def enclosed_by(self, other: _MaskOrEdge):
+    def enclosed_by(self, other: _MaskOrEdge) -> prp.Property:
         return _DualEdgeProperty(
             edge1=self, edge2=other, name="enclosed_by",
             commutative=False, allow_mask2=True,
         )
 
-    def interact_with(self, other: _MaskOrEdge):
+    def interact_with(self, other: _MaskOrEdge) -> "_Edge":
         return _DualEdgeOperation(
             edge1=self, edge2=other, name="interact_with", allow_mask2=True,
         )
 
 
 class _DualEdgeOperation(_Edge):
+    """`_DualEdgeOperation` represents the resulting `_Edge` from an
+    operation performed on a `_Edge` object and a `_Edge` or `Mask` object.
+    """
     def __init__(self, *,
         edge1: _Edge, edge2: _MaskOrEdge, name: str, allow_mask2: bool=False,
     ):
@@ -76,6 +88,9 @@ class _DualEdgeOperation(_Edge):
 
 
 class MaskEdge(_Edge):
+    """Objects of this class represent the edges of shapes present on a `Mask`
+    object.
+    """
     def __init__(self, mask: msk._Mask):
         self.mask = mask
 
@@ -83,17 +98,26 @@ class MaskEdge(_Edge):
 
 
 class Join(_Edge):
-    def __init__(self, edges: SingleOrMulti[_Edge].T):
-        self.edges = edges = _util.v2t(edges)
+    """`Joim` represent the resulting `_Edge` object from joining
+    the edges from two or more `_Edge` objects.
+    """
+    def __init__(self, edges: MultiT[_Edge]):
+        self.edges = edges = cast_MultiT(edges)
 
         super().__init__(name="join({})".format(",".join(str(edge) for edge in edges)))
 
 
 class Intersect(_Edge):
-    def __init__(self, edges: SingleOrMulti[_MaskOrEdge].T):
-        edges = _util.v2t(edges)
+    """`Joim` is the resulting `_Edge` object representing the overlapping
+    parts of the edges from two or more `_Edge` objects.
+
+    Crossing edges can result in points but the handling of this is application
+    dependent.
+    """
+    def __init__(self, edges: MultiT[_MaskOrEdge]):
+        edges = cast_MultiT(edges)
         if not any(isinstance(edge, _Edge) for edge in edges):
-            raise ValueError("at least one element of edges has to be of type 'Edge'")
+            raise TypeError("at least one element of edges has to be of type 'Edge'")
         self.edges = edges
 
         super().__init__(name="intersect({})".format(",".join(str(edge) for edge in edges)))
