@@ -10,6 +10,7 @@ Attributes:
 """
 import abc, enum
 from itertools import product
+from math import floor
 from typing import (
     Any, Dict, Iterable, Iterator, Generator, Collection, Tuple, List,
     Optional, Union, TypeVar, cast, overload
@@ -24,7 +25,8 @@ __all__ = [
     "epsilon",
     "Rotation", "FloatPoint",
     "RotationContext", "MoveContext",
-    "Point", "origin", "Line", "Polygon", "Rect", "MultiShape",
+    "Point", "origin", "Line", "Polygon", "Rect", "Ring", "RectRing",
+    "MultiPartShape", "MultiShape",
     "RepeatedShape", "ArrayShape",
     "MaskShape", "MaskShapes",
 ]
@@ -932,6 +934,170 @@ class Ring(Polygon):
         self.ring_width = ring_width
 
 
+class RectRing(_Shape):
+    """A `RectRing` object is a shape that consists of a ring of `Rect` objects.
+
+    An exception will be raised when there is not enough room to put the four corner
+    rects.  
+    If the 'Rect' objects needs to be on a grid all dimensions specified for this
+    object - including outer bound placement, width & height - have to be double that
+    grid number.
+
+    Arguments:
+        outer_bound: the outer bound of the ring; e.g. the generated rect shapes
+            will be inside and touching the bound.
+        rect_width: the width of the generated rect objects.
+        rect_height: the height of the generated rect objects; by default it will
+            be the same as rect_width.
+        min_rect_space: the minimum space between two rect structures.
+    """
+    # TODO: Describe rules to get shapes on grid
+    def __init__(self, *,
+        outer_bound: Rect,
+        rect_width: IntFloat, rect_height: Optional[IntFloat]=None,
+        min_rect_space: IntFloat,
+    ):
+        if rect_height is None:
+            rect_height = rect_width
+        if (outer_bound.width + epsilon) < (2*rect_width + min_rect_space):
+            raise ValueError(
+                "outer_bound width not big enough to fit two rects in"
+            )
+        if (outer_bound.height + epsilon) < (2*rect_height + min_rect_space):
+            raise ValueError(
+                "outer_bound height not big enough to fit two rects in"
+            )
+
+        self._outer_bound = outer_bound
+        self._rect_width = float(rect_width)
+        self._rect_height = float(rect_height)
+        self._min_rect_space = float(min_rect_space)
+
+        pitch_x = rect_width + min_rect_space
+        # Rects in horizontal direction besides corners
+        self._n_x = floor(
+            (
+                self.outer_bound.width
+                - (2*self.rect_width + self._min_rect_space)
+                + epsilon
+            )/pitch_x
+        )
+        assert self._n_x >= 0, "Internal error"
+
+        pitch_y = rect_height + min_rect_space
+        # Rects in vertical direction besides corners
+        self._n_y = floor(
+            (
+                self.outer_bound.height
+                - (2*self.rect_height + self._min_rect_space)
+                + epsilon
+            )/pitch_y
+        )
+        assert self._n_y >= 0, "Internal error"
+
+    @property
+    def outer_bound(self) -> Rect:
+        return self._outer_bound
+    @property
+    def rect_width(self) -> float:
+        return self._rect_width
+    @property
+    def rect_height(self) -> float:
+        return self._rect_height
+    @property
+    def min_rect_space(self) -> float:
+        return self._min_rect_space
+
+    def moved(self, *,
+        dxy: "Point", context: Optional[MoveContext]=None,
+    ) -> "RectRing":
+        return RectRing(
+            outer_bound=self.outer_bound.moved(dxy=dxy, context=context),
+            rect_width=self.rect_width, rect_height=self.rect_height,
+            min_rect_space=self.min_rect_space,
+        )
+
+    def rotated(self, *,
+        rotation: Rotation, context: Optional[RotationContext]=None,
+    ) -> "RectRing":
+        return RectRing(
+            outer_bound=self.outer_bound.rotated(rotation=rotation, context=context),
+            rect_width=self.rect_width, rect_height=self.rect_height,
+            min_rect_space=self.min_rect_space,
+        )
+
+    @property
+    def pointsshapes(self) -> Generator["_PointsShape", None, None]:
+        rect = Rect.from_size(width=self.rect_width, height=self.rect_height)
+        left = self.outer_bound.left
+        bottom = self.outer_bound.bottom
+        right = self.outer_bound.right
+        top = self.outer_bound.top
+
+        left_x = left + 0.5*self.rect_width
+        right_x = right - 0.5*self.rect_width
+        mid_x = self.outer_bound.center.x
+        bottom_y = bottom + 0.5*self.rect_height
+        top_y = top - 0.5*self.rect_height
+        mid_y = self.outer_bound.center.y
+
+        pitch_x = self.rect_width + self.min_rect_space
+        pitch_y = self.rect_height + self.min_rect_space
+
+        # corners
+        yield rect + Point(x=left_x, y=bottom_y)
+        yield rect + Point(x=left_x, y=top_y)
+        yield rect + Point(x=right_x, y=bottom_y)
+        yield rect + Point(x=right_x, y=top_y)
+
+        # bottom and top
+        left_x2 = mid_x - 0.5*(self._n_x - 1)*pitch_x
+        for n in range(self._n_x):
+            x = left_x2 + n*pitch_x
+            yield rect + Point(x=x, y=bottom_y)
+            yield rect + Point(x=x, y=top_y)
+
+        # left and right
+        bottom_y2 = mid_y - 0.5*(self._n_y - 1)*pitch_y
+        for n in range(self._n_y):
+            y = bottom_y2 + n*pitch_y
+            yield rect + Point(x=left_x, y=y)
+            yield rect + Point(x=right_x, y=y)
+
+    @property
+    def bounds(self):
+        return self.outer_bound
+
+    @property
+    def area(self) -> float:
+        return (4 + 2*self._n_x + 2*self._n_y)*self.rect_width*self.rect_height
+
+    def __eq__(self, o: object) -> bool:
+        if not isinstance(o, RectRing):
+            return False
+        else:
+            return all((
+                self.outer_bound == o.outer_bound,
+                abs(self.rect_width - o.rect_width) <= epsilon,
+                abs(self.rect_height - o.rect_height) <= epsilon,
+                abs(self.min_rect_space - o.min_rect_space) <= epsilon,
+            ))
+
+    def __hash__(self) -> int:
+        return hash(
+            (self.outer_bound, self.rect_width, self.rect_height, self.min_rect_space),
+        )
+
+    def __repr__(self) -> str:
+        s_args = ",".join((
+            f"outer_bound={self.outer_bound!r}",
+            f"rect_width={self.rect_width!r}",
+            f"rect_height={self.rect_height!r}",
+            f"min_rect_space={self.min_rect_space!r}",
+        ))
+        return f"RingRect({s_args})"
+
+
 class MultiPartShape(Polygon):
     """This shape represents a single polygon shape that consist of
     a build up of touching parts.
