@@ -1,7 +1,7 @@
 # SPDX-License-Identifier: GPL-2.0-or-later OR AGPL-3.0-or-later OR CERN-OHL-S-2.0+
 from math import floor, ceil
 import abc
-from typing import Optional, Union, cast, overload
+from typing import Optional, Union, Iterable, cast, overload
 
 from .. import _util
 from ..typing import IntFloat
@@ -15,12 +15,29 @@ __all__ = ["Technology"]
 
 
 class Technology(abc.ABC):
-    class TechnologyError(Exception):
-        pass
+    """A `Technology` object is the representation of a semiconductor process.
+    It's mainly a list of `_Primitive` objects with these primitives describing
+    the capabilities of the technolgy.
+
+    the `Technology` class is an abstract base class so subclasses need to
+    implement some of the abstract methods defined in this base class.
+
+    The setup of the primitives for a technology is done in the `_init()` method.
+    Typically subclasses should not overload the `__init__()` method.
+
+    Next to the `_init()` abstract method subclasses also need to define some
+    abstract properties to define some properties of the technology. These are
+    `name()`, `grid()` and `substrate_type()`.
+    """
     class ConnectionError(Exception):
         pass
 
     class _ComputedSpecs:
+        """`Technology._ComputedSpecs` is a helper class that allow to compute
+        some properties on a technology. This class should not be instantiated
+        by user code by is access through the `computed` attribute of a `Technology`
+        object.
+        """
         def __init__(self, tech: "Technology"):
             self.tech = tech
 
@@ -28,18 +45,35 @@ class Technology(abc.ABC):
         def min_space(self,
             primitive1: "prm._MaskPrimitive", primitive2: None=None,
         ) -> float:
-            ...
+            ... # pragma: no cover
         @overload
         def min_space(self,
             primitive1: "prm._Primitive", primitive2: "prm._Primitive",
         ) -> float:
-            ...
+            ... # pragma: no cover
         def min_space(self,
             primitive1: "prm._Primitive", primitive2: Optional["prm._Primitive"]=None,
         ) -> float:
+            """Compute the minimum space between one or two primitives. It will go
+            over all the rules to determine the minimum space for the provided
+            primitive.
+            This function allows to compute the min_space on primitives that are
+            derived from other primitives.
+
+            Arguments:
+                primitive1: the first primitive
+                    if primitive2 is not given, the minimum space between shapes
+                    on the primitive1 layer is returned.
+                primitive2: the optional second primitive.
+                    if specified the minimum space between shape on layer primitive1
+                    and shapes on layer primitive2 is returned.
+            """
             def check_prim(prim: prm.Spacing) -> bool:
                 if primitive2 is None:
-                    return (prim.primitives2 is None)
+                    return (
+                        (primitive1 in prim.primitives1)
+                        and (prim.primitives2 is None)
+                    )
                 elif prim.primitives2 is not None:
                     return (
                         (
@@ -62,7 +96,7 @@ class Technology(abc.ABC):
             if (primitive2 is None) or (primitive1 == primitive2):
                 try:
                     space = cast(prm._WidthSpacePrimitive, primitive1).min_space
-                except AttributeError:
+                except AttributeError: # pragma: no cover
                     pass
                 else:
                     spaces.append(space)
@@ -77,6 +111,19 @@ class Technology(abc.ABC):
         def min_width(self, primitive: "prm._WidthSpacePrimitive", *,
             up: bool=False, down: bool=False, min_enclosure: bool=False,
         ):
+            """Compute the minimum width of a primitive.
+            This method allows to take into account via enclosure rules to compute
+            minimum width but still be contacted through a via.
+
+            Arguments:
+                primitive: the primitive
+                up: wether it needs to take a connection from the top with a Via
+                    into account.
+                down: wether it needs to take a connection from the bottom with a Via
+                    into account.
+                min_enclosure: if True it will take the minimum value minimum value
+                    of the relevant via enclosure rule; otherwise the maximum value.
+            """
             assert primitive.min_width is not None, (
                 "primitive has to have the min_with attribute"
             )
@@ -105,27 +152,50 @@ class Technology(abc.ABC):
         def min_pitch(self, primitive: "prm._WidthSpacePrimitive", *,
             up: bool=False, down: bool=False, min_enclosure: bool=False,
         ):
+            """Compute the minimum pitch of a primitive.
+            It's the minimum width plus the minimum space.
+
+            Arguments:
+                primitive: the primitive
+                up: wether it needs to take a connection from the top with a Via
+                    into account.
+                down: wether it needs to take a connection from the bottom with a Via
+                    into account.
+                min_enclosure: if True it will take the minimum value minimum value
+                    of the relevant via enclosure rule; otherwise the maximum value.
+            """
             w = self.min_width(primitive, up=up, down=down, min_enclosure=min_enclosure)
             return w + primitive.min_space
 
     @property
     @abc.abstractmethod
     def name(self) -> str:
-        raise NotImplementedError
+        """property with the name of the technology"""
+        ... # pragma: no cover
     @property
     @abc.abstractmethod
     def grid(self) -> float:
-        raise NotImplementedError
+        """property with the minimum grid of the technology
+
+        Optionally primitives may define a bigger grid for their shapes.
+        """
+        ... # pragma: no cover
     @property
     @abc.abstractmethod
     def substrate_type(self) -> str:
-        raise NotImplementedError
+        """property with the substrate type of the technology
+
+        It has to be "n", "p" or "undoped".
+        """
+        ... # pragma: no cover
 
     def __init__(self):
         self._init_done = False
 
         if not self.substrate_type in ("n", "p", "undoped"):
-            raise ValueError("substrate_type Technology class attribute has to be 'n', 'p' or 'undoped'")
+            raise ValueError(
+                "substrate_type Technology class attribute has to be 'n', 'p' or 'undoped'",
+            )
 
         self._primitives = prims = prm.Primitives()
 
@@ -142,23 +212,38 @@ class Technology(abc.ABC):
         self.computed = self._ComputedSpecs(self)
 
     def is_ongrid(self, v: float, mult: int=1) -> float:
+        """Returns wether a value is on grid or not.
+
+        Arguments:
+            w: value to check
+            mult: value has to be on `mult*grid`
+        """
         g = mult*self.grid
         ng = round(v/g)
-        return abs(v - ng*g) < (g/100.0)
+        return abs(v - ng*g) < geo.epsilon
 
     @overload
     def on_grid(self,
         dim: IntFloat, *, mult: int=1, rounding: str="nearest",
     ) -> float:
-        ... # no cover
+        ... # pragma: no cover
     @overload
     def on_grid(self,
         dim: geo.Point, *, mult: int=1, rounding: str="nearest",
     ) -> geo.Point:
-        ... # no cover
+        ... # pragma: no cover
     def on_grid(self,
         dim: Union[IntFloat, geo.Point], *, mult: int=1, rounding: str="nearest",
     ) -> Union[float, geo.Point]:
+        """Compute a value on grid from a given value.
+
+        Arguments:
+            dim: value to put on grid
+            mult: value will be put on `mult*grid`
+            rounding: how to round the value
+                Has to be one of "floor", "nearest", "ceiling"; "nearest" is the
+                default.
+        """
         if isinstance(dim, geo.Point):
             return geo.Point(
                 x=self.on_grid(dim.x, mult=mult, rounding=rounding),
@@ -166,9 +251,9 @@ class Technology(abc.ABC):
             )
         dim = _util.i2f(dim)
         if rounding == "floor":
-            dim += 0.01*self.grid
+            dim += geo.epsilon
         elif rounding == "ceiling":
-            dim -= 0.01*self.grid
+            dim -= geo.epsilon
         else:
             if rounding != "nearest":
                 raise ValueError(
@@ -178,16 +263,18 @@ class Technology(abc.ABC):
         flookup = {"nearest": round, "floor": floor, "ceiling": ceil}
         try:
             f = flookup[rounding]
-        except KeyError:
-            raise ValueError(
-                f"rounding has to be one of {tuple(flookup.keys())}, not '{rounding}'"
-            )
+        except KeyError: # pragma: no cover
+            raise RuntimeError(f"Not implemeted: rounding '{rounding}'")
 
         return f(dim/(mult*self.grid))*mult*self.grid
 
     @property
     def dbu(self) -> float:
-        """Return database unit compatible with technology grid"""
+        """Returns database unit compatible with technology grid. An exception is
+        raised if the technology grid is not a multuple of 10pm.
+
+        This method is specifically for use to export to format that use the dbu.
+        """
         igrid = int(round(1e6*self.grid))
         assert (igrid%10) == 0
         if (igrid%100) != 0:
@@ -199,7 +286,16 @@ class Technology(abc.ABC):
 
     @abc.abstractmethod
     def _init(self):
-        raise RuntimeError("abstract base method _init() has to be implemnted in subclass")
+        """Abstract method that needs to setup the primitives.
+
+        When entering this function the object's `_primitives` attribute will be empty
+        and upon return from this method this attribute will have to be filled with
+        all the primitives of the given technology.
+
+        After the initialization has called this function it will perform connectivity
+        and other consistency checks on the provided list of primitives.
+        """
+        ... # pragma: no cover
 
     def _build_interconnect(self):
         prims = self._primitives
@@ -223,9 +319,11 @@ class Technology(abc.ABC):
         implants = set() # Implants to add
         markers = set() # Markers to add
         # the wells, fixed
+        deepwells = set(prims.__iter_type__(prm.DeepWell))
         wells = set(prims.__iter_type__(prm.Well))
 
         # Wells are the first primitives in line
+        add_prims(sorted(deepwells, key=get_name))
         add_prims(sorted(wells, key=get_name))
 
         # process waferwires
@@ -255,7 +353,7 @@ class Technology(abc.ABC):
 
         def allwires(wire):
             if isinstance(wire, prm.Resistor):
-                yield allwires(wire.wire)
+                yield from allwires(wire.wire)
                 yield from wire.indicator
             if isinstance(wire, prm._PinAttribute) and wire.pin is not None:
                 yield from wire.pin
@@ -273,10 +371,15 @@ class Technology(abc.ABC):
                     viabottoms.update(via.bottom)
                     viatops.update(via.top)
 
-                noconn = viabottoms - bottomwires
+                noconn = tuple(filter(
+                    # MIMTop does not need to be connected from bottom
+                    lambda l: not isinstance(l, prm.MIMTop),
+                    bottomwires - viabottoms,
+                ))
                 if noconn:
                     raise Technology.ConnectionError(
-                        f"wires ({', '.join(wire.name for wire in noconn)}) not connected from bottom"
+                        f"wires ({', '.join(wire.name for wire in noconn)})"
+                        " not in bottom list of any via"
                     )
 
                 for bottom in viabottoms:
@@ -293,7 +396,8 @@ class Technology(abc.ABC):
 
         if vias:
             raise Technology.ConnectionError(
-                f"vias ({', '.join(via.name for via in vias)}) not connected to bottom wires"
+                f"vias ({', '.join(via.name for via in vias)}) have no connection to"
+                " a technology bottom wire"
             )
 
         # Add via and it's blockage layers
@@ -305,9 +409,14 @@ class Technology(abc.ABC):
         # process mosfets
         mosfets = set(prims.__iter_type__(prm.MOSFET))
         gates = {mosfet.gate for mosfet in mosfets}
+        allgates = set(prims.__iter_type__(prm.MOSFETGate))
+        if gates != allgates:
+            diff = allgates - gates
+            if diff:
+                raise prm.UnusedPrimitiveError(diff.pop())
+            raise RuntimeError("Unhandled error condition") # pragma: no cover
         actives = {gate.active for gate in gates}
         polys = {gate.poly for gate in gates}
-        bottomwires.update(polys) # Also need to be connected
         for mosfet in mosfets:
             implants.update(mosfet.implant)
             if mosfet.well is not None:
@@ -339,6 +448,14 @@ class Technology(abc.ABC):
         for resistor in resistors:
             markers.update(resistor.indicator)
 
+        # process capacitors
+        mimtops = set(prims.__iter_type__(prm.MIMTop))
+        mimcaps = tuple(prims.__iter_type__(prm.MIMCapacitor))
+        usedtops = set(c.top for c in mimcaps)
+        unusedtops = mimtops - usedtops
+        if unusedtops:
+            raise prm.UnusedMIMTop(unusedtops.pop())
+
         # process diodes
         diodes = set(prims.__iter_type__(prm.Diode))
         for diode in diodes:
@@ -352,7 +469,7 @@ class Technology(abc.ABC):
         # extra rules
         rules = set(prims.__iter_type__(prm._RulePrimitive))
 
-        add_prims((*markers, *resistors, *diodes, *bipolars, *rules))
+        add_prims((*markers, *resistors, *mimcaps, *diodes, *bipolars, *rules))
 
         # process auxiliary
         def aux_key(aux: prm.Auxiliary) -> str:
@@ -391,7 +508,14 @@ class Technology(abc.ABC):
         rules._freeze_()
 
     @property
-    def substrate(self):
+    def substrate(self) -> msk._Mask:
+        """Property representing the substrate of the technology; it's defined as the area
+        that is outside any of the wells of the technology.
+
+        As this value needs access to the list of wells it's only available afcer the
+        technology has been initialized and is not available during run of the `_init()`
+        method.
+        """
         if not self._init_done:
             raise AttributeError("substrate may not be accessed during object initialization")
         if self._substrate is None:
@@ -405,15 +529,21 @@ class Technology(abc.ABC):
         return self._substrate
 
     @property
-    def rules(self):
+    def rules(self) -> Iterable[rle._Rule]:
+        """Return all the rules that are derived from the primitives of the technology."""
         return self._rules
 
     @property
-    def primitives(self):
+    def primitives(self) -> "prm.Primitives":
+        """Return the primitives of the technology."""
         return self._primitives
 
     @property
-    def designmasks(self):
+    def designmasks(self) -> Iterable[msk.DesignMask]:
+        """Return all the `DesignMask`s defined by the primitives of the technology
+
+        The property makes sure there are no duplicates in the returned iterable.
+        """
         masks = set()
         for prim in self._primitives:
             for mask in prim.designmasks:
