@@ -66,6 +66,9 @@ class GeometryTest(unittest.TestCase):
         self.assertEqual(r_r, r_ms.shape)
         self.assertEqual(r_ms, r_mss[0])
 
+    # RotationContext and MoveContext are tested from from the layout
+    # unit tests.
+
     def test_abstract(self):
         with self.assertRaisesRegex(
             TypeError, "^Can't instantiate abstract class _Shape",
@@ -450,6 +453,71 @@ class GeometryTest(unittest.TestCase):
         ))
         self.assertEqual(ring, polygon)
 
+    def test_rectring(self):
+        bb = _geo.Rect.from_size(center=_geo.Point(x=1.0, y=1.5), width=2, height=3)
+        bb2 = _geo.Rect.from_size(center=_geo.Point(x=1.5, y=1.0), width=3, height=2)
+        dxy = _geo.Point(x=1.0, y=1.0)
+
+        with self.assertRaises(ValueError):
+            _geo.RectRing(outer_bound=bb, rect_width=1.0, min_rect_space=1.0)
+        with self.assertRaises(ValueError):
+            _geo.RectRing(outer_bound=bb2, rect_width=1.0, min_rect_space=1.0)
+
+        ring1 = _geo.RectRing(outer_bound=bb, rect_width=0.5, min_rect_space=0.5)
+        ring2 = _geo.RectRing(
+            outer_bound=bb, rect_width=0.5, rect_height=0.5, min_rect_space=0.5,
+        )
+        ring3 = _geo.RectRing(outer_bound=bb, rect_width=0.5, min_rect_space=1.0)
+        ring4 = _geo.RectRing(outer_bound=bb2, rect_width=0.5, min_rect_space=0.5)
+        ring5 = _geo.RectRing(outer_bound=(bb + dxy), rect_width=0.5, min_rect_space=0.5)
+        ring6 = _geo.RectRing(
+            outer_bound=_geo.Rotation.MX*bb, rect_width=0.5, min_rect_space=0.5,
+        )
+
+        self.assertEqual(ring1, ring2)
+        self.assertNotEqual(ring1, ring3)
+        self.assertNotEqual(ring1, ring4)
+
+        self.assertNotEqual(ring1, 1.0)
+
+        self.assertEqual(hash(ring1), hash(ring2))
+        self.assertEqual(repr(ring1), repr(ring2))
+        self.assertNotEqual(hash(ring1), hash(ring3))
+
+        self.assertEqual(ring1.bounds, bb)
+
+        self.assertEqual(ring1 + dxy, ring5)
+        self.assertEqual(_geo.Rotation.MX*ring1, ring6)
+
+        rect = _geo.Rect.from_size(width=0.5, height=0.5)
+        self.assertEqual(
+            set(ring1.pointsshapes),
+            set(
+                rect + p for p in (
+                    _geo.Point(x=0.25, y=0.25),
+                    _geo.Point(x=0.25, y=1.50),
+                    _geo.Point(x=0.25, y=2.75),
+                    _geo.Point(x=1.75, y=0.25),
+                    _geo.Point(x=1.75, y=1.50),
+                    _geo.Point(x=1.75, y=2.75),
+                )
+            )
+        )
+        self.assertEqual(
+            set(ring4.pointsshapes),
+            set(
+                rect + p for p in (
+                    _geo.Point(x=0.25, y=0.25),
+                    _geo.Point(x=0.25, y=1.75),
+                    _geo.Point(x=1.50, y=0.25),
+                    _geo.Point(x=1.50, y=1.75),
+                    _geo.Point(x=2.75, y=0.25),
+                    _geo.Point(x=2.75, y=1.75),
+                )
+            )
+        )
+        self.assertEqual(ring1.area, 6*rect.area)
+
     def test_multipartshape(self):
         r_all = _geo.Rect(left=-2.0, bottom=-1.0, right=1.0, top=1.0)
         r_left = _geo.Rect(left=-2.0, bottom=-1.0, right=0.0, top=1.0)
